@@ -38,10 +38,10 @@ def cast(cls, x, implicit_conversions=None):
             raise TypeError(f"{x}: {type(x)} is not compatible with {cls}")
         fields = cls.__annotations__
         if cls.__total__:
-            if set(fields.keys()) != set(x.keys()):
+            if set(fields) != set(x):
                 raise TypeError(f"{x}: {type(x)} is not compatible with {cls}")
         else:
-            if not set(x.keys()).issubset(set(fields.keys())):
+            if not set(x).issubset(set(fields)):
                 raise TypeError(f"{x}: {type(x)} is not compatible with {cls}")
         return {
             k: cast(fields[k], v, implicit_conversions=implicit_conversions)
@@ -61,7 +61,7 @@ def cast(cls, x, implicit_conversions=None):
         if not isinstance(x, (int, float)):
             raise TypeError(f"{x}: {type(x)} is not compatible with {cls}")
         return x
-    elif type(cls) == type:
+    elif isinstance(cls, type):
         if not isinstance(x, cls):
             raise TypeError(f"{x}: {type(x)} is not compatible with {cls}")
         return x
@@ -69,10 +69,22 @@ def cast(cls, x, implicit_conversions=None):
         if x not in cls.__args__:
             raise TypeError(f"{x} is not compatible with {cls}")
         return x
-    elif cls.__origin__ == list or cls.__origin__ == collections.abc.Sequence:
+    elif cls.__origin__ == set or cls.__origin__ in (
+        collections.abc.Set,
+        collections.abc.MutableSet,
+    ):
+        vcls = cls.__args__[0]
+        return set(cast(vcls, v, implicit_conversions=implicit_conversions) for v in x)
+    elif cls.__origin__ == list or cls.__origin__ in (
+        collections.abc.Sequence,
+        collections.abc.MutableSequence,
+    ):
         vcls = cls.__args__[0]
         return [cast(vcls, v, implicit_conversions=implicit_conversions) for v in x]
-    elif cls.__origin__ == dict or cls.__origin__ == collections.abc.Mapping:
+    elif cls.__origin__ == dict or cls.__origin__ in (
+        collections.abc.Mapping,
+        collections.abc.MutableMapping,
+    ):
         kcls, vcls = cls.__args__
         return {
             cast(kcls, k, implicit_conversions=implicit_conversions): cast(
