@@ -42,7 +42,7 @@ def cast(cls, x, implicit_conversions=None):
         if not isinstance(x, (int, float)):
             raise TypeError(f"{x}: {type(x)} is not compatible with {cls}")
         return x
-    elif type(cls) == type:
+    elif isinstance(cls, type):
         if not isinstance(x, cls):
             raise TypeError(f"{x}: {type(x)} is not compatible with {cls}")
         return x
@@ -50,10 +50,22 @@ def cast(cls, x, implicit_conversions=None):
         if x not in cls:
             raise TypeError(f"{x} is not compatible with {cls}")
         return x
-    elif cls.__origin__ == list or cls.__origin__ == collections.abc.Sequence:
+    elif cls.__origin__ == set or cls.__origin__ in (
+        collections.abc.Set,
+        collections.abc.MutableSet,
+    ):
+        vcls = cls.__args__[0]
+        return set(cast(vcls, v, implicit_conversions=implicit_conversions) for v in x)
+    elif cls.__origin__ == list or cls.__origin__ in (
+        collections.abc.Sequence,
+        collections.abc.MutableSequence,
+    ):
         vcls = cls.__args__[0]
         return [cast(vcls, v, implicit_conversions=implicit_conversions) for v in x]
-    elif cls.__origin__ == dict or cls.__origin__ == collections.abc.Mapping:
+    elif cls.__origin__ == dict or cls.__origin__ in (
+        collections.abc.Mapping,
+        collections.abc.MutableMapping,
+    ):
         kcls, vcls = cls.__args__
         return {
             cast(kcls, k, implicit_conversions=implicit_conversions): cast(
